# 2012 Sim Ayers
# version 1.0
# modified from draw_angle_dim.rb
# Copyright  2011 Stephen Baumgartner <steve@slbaumgartner.com>
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# 
#class DrawArcTool

module ROOFFRAMING

class DrawArcTool < TetrahedronTool
  
  def initialize
    # @ip is the InputPoint from the current pick
    @ip = Sketchup::InputPoint.new
    # @ip1 is the InputPoint from the previous pick
    # this is used as reference when @ip picks, which, in principle, should
    # cause the inference engine to favor points on the edge containing the 
    # previous pick
    @ip1 = Sketchup::InputPoint.new
    
    @drawn = false
    
    # @pts[0] = center point of arc
    # @pts[1] = radius of arc
    # @pts[2] = ending angle of arc
    @pts = []
    
    # @state = 0 => waiting for first pick
    # @state = 1 => waiting for vertex pick
    # @state = 2 => waiting for second pick
    # @state = 3 => have three points, ready to draw
    @state = 0
    
    # radius for drawn arc can be calculated by default or input via VCB
    @user_radius = -1
    @radius = 0
     # tab key toggles between drawing inside and outside angle dimension
    @inside = true
   
  end
  
  def show_status
    Sketchup::set_status_text "arc radius", SB_VCB_LABEL
    if(@radius != 0)
      Sketchup::set_status_text Sketchup.format_length(@radius), SB_VCB_VALUE
      else
      Sketchup::set_status_text "", SB_VCB_VALUE
    end
    case @state
      when 0
      Sketchup::set_status_text "Point 1: center point of arc"
      when 1
      Sketchup::set_status_text "Point 2: starting point of arc"
      when 2
      Sketchup::set_status_text "Point 3: end point of arc"
    end
    
  end
  
  # clear out all saved data from previous operation
  def reset
    @pts = []
    @state = 0
    
    @ip1.clear
    @drawn = false
    show_status
  end
  
  def activate
    reset
  end
  
  def deactivate(view)
    reset
    view.invalidate if @drawn
  end
  
  def set_current_point(x, y, view)
    case @state
      when 0
      # capture center point of arc
      @pts[0] = @ip.position
      # and cancel user's ability to rescale last drawn angle dim
      @drawn = false
      when 1
      # capture radius point
      @pts[1] = @ip.position
      # tell the user what radius arc will be drawn, so they can change via VCB if desired
      length = @pts[0].distance @pts[1]
      if(@user_radius > 0)
        @radius = @user_radius
      end
      when 2
      # capture third point
      @pts[2] = @ip.position
    end
    show_status
  end
  
  # need this to cause inference point and tooltips to be drawn
  def onMouseMove(flags, x, y, view)
    @ip.pick(view, x, y, @ip1)
    view.invalidate
  end
  
  # draw the angle dimension info
  def draw_arc
    model = Sketchup.active_model
    # this enables undo of the whole operation as a unit
    model.start_operation "Arc"
    @radius = @pts[0].distance @pts[1]
    # make sure we are using the user's selected radius regardless of
    # which state this object was in when it was selected.
    if(@user_radius > 0)
      @radius = @user_radius
    end
     
    # make vectors from the user's input points.  These vectors point
    # out from the angle vertex along the two edges.  They are needed by
    # some subsequent methods that require directions for offsets, not
    # points
    vec1 = @pts[0].vector_to @pts[1]
    vec2 = @pts[0].vector_to @pts[2]
    
    # trap degenerate cases
    if(vec1.length == 0 or vec2.length == 0)
      UI.messagebox("You must select three distinct points!")
      reset
      return
    end
    if(vec1.parallel? vec2)
      UI.messagebox("The selected lines are parallel -\nthe angle is 0 or 180")
      reset
      return
    end
    
        
    # scale the vectors to extend 1.5 the radius.  This will affect
    # the drawn edges.  The value 1.5 is arbitrary - change it if another
    # look is desired
    vec1.length = @pts[0].distance @pts[1]
    vec2.length = @pts[0].distance @pts[2]   
    
    # find the angle between the vectors and the normal to their plane
    # this calculation should not explode, since we trapped the case of
    # parallel above (angle = 0 or 180), but the calculation of the normal
    # will be numerically unstable when angle is very small or very close to
    # 180.  I have tested at 0.1 degrees without problems.
    angle = vec1.angle_between vec2
    complement = 360.degrees - angle
    normal = (vec1 * vec2).normalize

    # create a new group in which we will draw our dimension entities
    model_ents = model.active_entities
    group = model_ents.add_group
    group.name = "Arc"
    ents = group.entities
      

    if(@inside)
      # interior angle mode
      # draw the arc across the angle at the selected radius
      arc = ents.add_arc @pts[0], vec1, normal, @radius, 0, angle, 30
           
      else
      # exterior angle mode
      arc2 = ents.add_arc @pts[0], vec1, normal, @radius, 0,-complement, 30
     
    end
    
    # tell undo the end of the bundled operation
    model.commit_operation
    
    #start over
    @drawn = true
    @state = 0
    show_status
  end
  
  # advance to the next state and set the status texts appropriately.
  def increment_state
    @state += 1
    case @state
      when 1
      # have first pick, ready for second
      # retain InputPoint as a hint for next
      @ip1.copy! @ip
      Sketchup::set_status_text "Point 2: is radius"
      when 2
      # have second pick, ready for third
      # retain InputPoint as a hint for next
      @ip1.copy! @ip
      Sketchup::set_status_text "Point 3: end angle of arc"
      when 3
      # have three picks, ready to draw the angle dimension
      draw_arc
    end
  end
  
  # user clicks the mouse button - capture the data point and advance the state
  def onLButtonDown(flags, x, y, view)
    set_current_point(x, y, view)
    increment_state
  end
  
  def onCancel(flag, view)
    view.invalidate if @drawn
    reset
  end
  
  # accept user input in the VCB as the desired radius of the dimension arc
  def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
      value = text.to_l
      rescue
      # Error parsing the text
      UI.messagebox("please enter a number for arc radius")
      value = nil
      Sketchup::set_status_text "", SB_VCB_VALUE
    end
    if(value <= 0.0)
      UI.messagebox("arc radius must be a positive number")
      value = nil
      Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    @user_radius = value
    
    # redraw at new radius, if appropriate
    if @drawn
      # undo the previous operation before redrawing
      # this avoids buildup of obsolete operations on the
      # undo stack
      Sketchup.undo if @state==0
      draw_angle_dim
    end
  end
  
  # invoked by SketchUp when the view is invalidated.  This makes sure the
  # pick point and tooltip are visible.
  def draw(view)
    view.tooltip = @ip.tooltip
    @ip.draw view
  end
  
    
  def onKeyDown(key, rpt, flags, view)
    # On a PC, key 9 is TAB.  SketchUp has no symbolic constant for this
    # If tab doesn't work, you can use the following to print to the
    # Ruby Console and see what the key code is on your system
    # puts "Key Pressed was " + key.to_s
    if key == 9
      @inside = !@inside
           
      # redraw in new mode if appropriate
      if @drawn
        # undo the previous operation before redrawing
        # this avoids buildup of unneeded operations on
        # the undo stack        
        Sketchup.undo if @state==0
        draw_arc
       
      end
    end
  end
  
  def onKeyUp(key, rpt, flags, view)
  end

end # class RoofAngleDimTool

end #module ROOFFRAMING

file_loaded("RoofFraming/draw_arc.rb")



